@@ -256,6 +256,20 @@ def test_lang():
     ('input:where([type="text"])', ['text-checked']),
     ('div:where(:not(#outer-div))', ['li-div', 'foobar-div']),
     ('div:where(div::before)', []),
+    ('p:has(input)', ['paragraph']),
+    ('p:has(fieldset input)', ['paragraph']),
+    ('p:has(> fieldset)', ['paragraph']),
+    ('ol:has(> div)', []),
+    ('ol:has(input, li)', ['first-ol']),
+    ('ol:has(input, fieldset)', []),
+    ('ol:has(+ p)', ['first-ol']),
+    ('ol:has(~ ol)', ['first-ol']),
+    ('ol:has(>a, ~ ol)', ['first-ol']),
+    ('ol:has(a,ol,  li  )', ['first-ol']),
+    ('ol:has(*)', ['first-ol']),
+    ('ol:has(:not(li))', ['first-ol']),
+    ('ol:has( > :not( li ))', []),
+    ('ol:has(:not(li, div))', []),
 
     # Invalid characters in XPath element names, should not crash
     (r'di\a0 v', []),
